@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2018 Savoir-faire Linux Inc.
+ *  Copyright (C) 2018-2019 Savoir-faire Linux Inc.
  *
  *  Author: Adrien Béraud <adrien.beraud@savoirfairelinux.com>
  *
@@ -20,7 +20,6 @@
 package cx.ring.services;
 
 import android.content.Context;
-import android.content.res.Configuration;
 import android.graphics.ImageFormat;
 import android.graphics.Point;
 import android.graphics.SurfaceTexture;
@@ -48,13 +47,16 @@ import android.util.Range;
 import android.util.Size;
 import android.view.Surface;
 import android.view.SurfaceHolder;
+import android.view.TextureView;
 
+import java.nio.ByteBuffer;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.List;
 
 import androidx.annotation.NonNull;
+import androidx.annotation.Nullable;
 import androidx.annotation.RequiresApi;
 import cx.ring.daemon.IntVect;
 import cx.ring.daemon.RingserviceJNI;
@@ -65,12 +67,14 @@ import cx.ring.views.AutoFitTextureView;
 @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
 class CameraServiceCamera2 extends CameraService {
     private static final String TAG = CameraServiceCamera2.class.getName();
-    private static final boolean USE_HARDWARE_ENCODER = false;
+    private static final int FPS_MAX = 30;
+    private static final int FPS_TARGET = 15;
 
     private final HandlerThread t = new HandlerThread("videoHandler");
 
     private final CameraManager manager;
     private CameraDevice previewCamera;
+    private MediaCodec currentCodec;
 
     CameraServiceCamera2(@NonNull Context c) {
         manager = (CameraManager) c.getSystemService(Context.CAMERA_SERVICE);
@@ -82,78 +86,92 @@ class CameraServiceCamera2 extends CameraService {
         if (manager == null)
             return;
         try {
+            cameras.clear();
             for (String id : manager.getCameraIdList()) {
                 currentCamera = id;
-                RingserviceJNI.addVideoDevice(id);
                 CameraCharacteristics cc = manager.getCameraCharacteristics(id);
                 int facing = cc.get(CameraCharacteristics.LENS_FACING);
                 if (facing == CameraCharacteristics.LENS_FACING_FRONT) {
                     cameraFront = id;
-                } else if (facing == CameraCharacteristics.LENS_FACING_BACK) {
-                    cameraBack = id;
-                } else if (facing == CameraCharacteristics.LENS_FACING_EXTERNAL) {
-                    cameraExternal = id;
                 }
+                cameras.add(id);
+                RingserviceJNI.addVideoDevice(id);
             }
             if (!TextUtils.isEmpty(cameraFront))
                 currentCamera = cameraFront;
-            RingserviceJNI.setDefaultDevice(currentCamera);
+            else if (!cameras.isEmpty())
+                currentCamera = cameras.get(0);
+
+            if (currentCamera != null) {
+                for (int i=0; i<cameras.size(); i++)
+                    if (cameras.get(i) == currentCamera) {
+                        currentCameraIndex = i;
+                        break;
+                    }
+                RingserviceJNI.setDefaultDevice(currentCamera);
+            }
         } catch (Exception e) {
             Log.w(TAG, "initVideo: can't enumerate devices", e);
         }
     }
 
     @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
-    private void listSupportedCodecs() {
-        int numCodecs = MediaCodecList.getCodecCount();
-        for (int i = 0; i < numCodecs; i++) {
-            try {
-                MediaCodecInfo codecInfo = MediaCodecList.getCodecInfoAt(i);
+    private void listSupportedCodecs(MediaCodecList list) {
+        try {
+            for (MediaCodecInfo codecInfo : list.getCodecInfos()) {
                 for (String type : codecInfo.getSupportedTypes()) {
-                    MediaCodecInfo.CodecCapabilities codecCaps = codecInfo.getCapabilitiesForType(type);
-                    MediaCodecInfo.EncoderCapabilities caps = codecCaps.getEncoderCapabilities();
-                    if (caps == null)
-                        continue;
-                    MediaCodecInfo.VideoCapabilities video_caps = codecCaps.getVideoCapabilities();
-                    if (video_caps == null)
-                        continue;
-                    Log.w(TAG, "Codec info:" + codecInfo.getName() + " type: " + type);
-                    Log.w(TAG, "Encoder capabilities: complexityRange: " + caps.getComplexityRange());
-                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P) {
-                        Log.w(TAG, "Encoder capabilities: qualityRange: " + caps.getQualityRange());
-                    }
-                    Log.w(TAG, "Encoder capabilities: VBR: " + caps.isBitrateModeSupported(MediaCodecInfo.EncoderCapabilities.BITRATE_MODE_VBR));
-                    Log.w(TAG, "Encoder capabilities: CBR: " + caps.isBitrateModeSupported(MediaCodecInfo.EncoderCapabilities.BITRATE_MODE_CBR));
-                    Log.w(TAG, "Encoder capabilities: CQ: " + caps.isBitrateModeSupported(MediaCodecInfo.EncoderCapabilities.BITRATE_MODE_CQ));
-                    Log.w(TAG, "Bitrate range: " + video_caps.getBitrateRange());
+                    try {
+                        MediaCodecInfo.CodecCapabilities codecCaps = codecInfo.getCapabilitiesForType(type);
+                        MediaCodecInfo.EncoderCapabilities caps = codecCaps.getEncoderCapabilities();
+                        if (caps == null)
+                            continue;
+                        MediaCodecInfo.VideoCapabilities video_caps = codecCaps.getVideoCapabilities();
+                        if (video_caps == null)
+                            continue;
+                        Log.w(TAG, "Codec info:" + codecInfo.getName() + " type: " + type);
+                        Log.w(TAG, "Encoder capabilities: complexityRange: " + caps.getComplexityRange());
+                        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P) {
+                            Log.w(TAG, "Encoder capabilities: qualityRange: " + caps.getQualityRange());
+                        }
+                        Log.w(TAG, "Encoder capabilities: VBR: " + caps.isBitrateModeSupported(MediaCodecInfo.EncoderCapabilities.BITRATE_MODE_VBR));
+                        Log.w(TAG, "Encoder capabilities: CBR: " + caps.isBitrateModeSupported(MediaCodecInfo.EncoderCapabilities.BITRATE_MODE_CBR));
+                        Log.w(TAG, "Encoder capabilities: CQ: " + caps.isBitrateModeSupported(MediaCodecInfo.EncoderCapabilities.BITRATE_MODE_CQ));
+                        Log.w(TAG, "Bitrate range: " + video_caps.getBitrateRange());
+                        for (int format : codecCaps.colorFormats) {
+                            Log.w(TAG, "Supported color format: " + format);
+                        }
 
-                    Range<Integer> widths = video_caps.getSupportedWidths();
-                    Range<Integer> heights = video_caps.getSupportedHeights();
-                    Log.w(TAG, "Supported sizes: " + widths.getLower() + "x" + heights.getLower() + " -> " + widths.getUpper() + "x" + heights.getUpper());
-                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
-                        Log.w(TAG, "AchievableFrameRates: " + video_caps.getAchievableFrameRatesFor(widths.getUpper(), heights.getUpper()));
-                    }
-                    Log.w(TAG, "SupportedFrameRates: " + video_caps.getSupportedFrameRatesFor(widths.getUpper(), heights.getUpper()));
+                        Range<Integer> widths = video_caps.getSupportedWidths();
+                        Range<Integer> heights = video_caps.getSupportedHeights();
+                        Log.w(TAG, "Supported sizes: " + widths.getLower() + "x" + heights.getLower() + " -> " + widths.getUpper() + "x" + heights.getUpper());
+                        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
+                            Log.w(TAG, "AchievableFrameRates: " + video_caps.getAchievableFrameRatesFor(1920, 1080));
+                        }
+                        Log.w(TAG, "SupportedFrameRates: " + video_caps.getSupportedFrameRatesFor(/*widths.getUpper(), heights.getUpper()*/1920, 1080));
 
-                    for (MediaCodecInfo.CodecProfileLevel profileLevel : codecCaps.profileLevels)
-                        Log.w(TAG, "profileLevels: " + profileLevel);
-                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
-                        Log.w(TAG, "FEATURE_IntraRefresh: " + codecCaps.isFeatureSupported(MediaCodecInfo.CodecCapabilities.FEATURE_IntraRefresh));
+                        for (MediaCodecInfo.CodecProfileLevel profileLevel : codecCaps.profileLevels)
+                            Log.w(TAG, "profileLevels: " + profileLevel);
+                        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
+                            Log.w(TAG, "FEATURE_IntraRefresh: " + codecCaps.isFeatureSupported(MediaCodecInfo.CodecCapabilities.FEATURE_IntraRefresh));
+                        }
+                    } catch (Exception e) {
+                        Log.w(TAG, "Can't query codec info", e);
                     }
                 }
-            } catch (Exception e) {
-                Log.w(TAG, "Can't query codec info", e);
             }
+        } catch (Exception e) {
+            Log.w(TAG, "Can't query codec info", e);
         }
     }
 
-    private Pair<MediaCodec, Surface> openCameraWithEncoder(VideoParams videoParams, String mimeType) {
-        final int BITRATE = 1600 * 1024;
+    private Pair<MediaCodec, Surface> openCameraWithEncoder(VideoParams videoParams, String mimeType, Handler handler, boolean isHd) {
+        final int BITRATE = isHd ? 192 * 8 * 1024 : 100 * 8 * 1024;
         MediaFormat format = MediaFormat.createVideoFormat(mimeType, videoParams.width, videoParams.height);
         format.setInteger(MediaFormat.KEY_MAX_INPUT_SIZE, 0);
         format.setInteger(MediaFormat.KEY_BIT_RATE, BITRATE);
         format.setInteger(MediaFormat.KEY_COLOR_FORMAT, MediaCodecInfo.CodecCapabilities.COLOR_FormatSurface);
-        format.setInteger(MediaFormat.KEY_FRAME_RATE, 30);
+        if (Build.VERSION.SDK_INT != Build.VERSION_CODES.LOLLIPOP)
+            format.setInteger(MediaFormat.KEY_FRAME_RATE, 24);
         /*if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
             format.setInteger(MediaFormat.KEY_I_FRAME_INTERVAL, -1);
             format.setInteger(MediaFormat.KEY_INTRA_REFRESH_PERIOD, 5);
@@ -163,7 +181,12 @@ class CameraServiceCamera2 extends CameraService {
         format.setInteger(MediaFormat.KEY_I_FRAME_INTERVAL, 5);
         //format.setInteger(MediaFormat.KEY_BITRATE_MODE, MediaCodecInfo.CodecCapabilities.BITRATE_MODE_VBR);
 
-        String codecName = new MediaCodecList(MediaCodecList.REGULAR_CODECS).findEncoderForFormat(format);
+        MediaCodecList codecs = new MediaCodecList(MediaCodecList.REGULAR_CODECS);
+        //listSupportedCodecs(codecs);
+        String codecName = codecs.findEncoderForFormat(format);
+
+        format.setInteger(MediaFormat.KEY_FRAME_RATE, 24);
+
         Surface encoderInput = null;
         MediaCodec codec = null;
         if (codecName != null) {
@@ -174,16 +197,22 @@ class CameraServiceCamera2 extends CameraService {
                 codec.setParameters(params);
                 codec.configure(format, null, null, MediaCodec.CONFIGURE_FLAG_ENCODE);
                 encoderInput = codec.createInputSurface();
-                codec.setCallback(new MediaCodec.Callback() {
+                MediaCodec.Callback callback = new MediaCodec.Callback() {
                     @Override
                     public void onInputBufferAvailable(@NonNull MediaCodec codec, int index) {
                     }
 
                     @Override
                     public void onOutputBufferAvailable(@NonNull MediaCodec codec, int index, @NonNull MediaCodec.BufferInfo info) {
-                        /*ByteBuffer buffer = codec.getOutputBuffer(index);
-                        RingserviceJNI.captureVideoPacket(buffer, info.size, info.offset, (info.flags & MediaCodec.BUFFER_FLAG_KEY_FRAME) != 0, info.presentationTimeUs);*/
-                        codec.releaseOutputBuffer(index, false);
+                        try {
+                            if ((info.flags & MediaCodec.BUFFER_FLAG_END_OF_STREAM) == 0) {
+                                ByteBuffer buffer = codec.getOutputBuffer(index);
+                                RingserviceJNI.captureVideoPacket(buffer, info.size, info.offset, (info.flags & MediaCodec.BUFFER_FLAG_KEY_FRAME) != 0, info.presentationTimeUs, videoParams.rotation);
+                                codec.releaseOutputBuffer(index, false);
+                            }
+                        } catch (IllegalStateException e) {
+                            Log.e(TAG, "MediaCodec can't process buffer", e);
+                        }
                     }
 
                     @Override
@@ -195,7 +224,13 @@ class CameraServiceCamera2 extends CameraService {
                     public void onOutputFormatChanged(@NonNull MediaCodec codec, @NonNull MediaFormat format) {
                         Log.e(TAG, "MediaCodec onOutputFormatChanged " + format);
                     }
-                });
+                };
+
+                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
+                    setCodecCallback(codec, callback, handler);
+                } else {
+                    setCodecCallback(codec, callback);
+                }
             } catch (Exception e) {
                 Log.e(TAG, "Can't open codec", e);
                 codec = null;
@@ -205,30 +240,51 @@ class CameraServiceCamera2 extends CameraService {
         return new Pair<>(codec, encoderInput);
     }
 
+    private void setCodecCallback(@NonNull MediaCodec codec, MediaCodec.Callback callback) {
+        codec.setCallback(callback);
+    }
+
+    @RequiresApi(api = Build.VERSION_CODES.M)
+    private void setCodecCallback(@NonNull MediaCodec codec, MediaCodec.Callback callback, Handler handler) {
+        codec.setCallback(callback, handler);
+    }
+
     /**
      * Compares two {@code Size}s based on their areas.
      */
     static class CompareSizesByArea implements Comparator<Size> {
-
         @Override
         public int compare(Size lhs, Size rhs) {
             // We cast here to ensure the multiplications won't overflow
             return Long.signum((long) lhs.getWidth() * lhs.getHeight() -
                     (long) rhs.getWidth() * rhs.getHeight());
         }
-
     }
 
-    private static Size chooseOptimalSize(Size[] choices, int textureViewWidth,
-                                          int textureViewHeight, int maxWidth, int maxHeight, Size aspectRatio) {
+    public void requestKeyFrame() {
+        Log.w(TAG, "requestKeyFrame()");
+        try {
+            if (currentCodec != null) {
+                Bundle params = new Bundle();
+                params.putInt(MediaCodec.PARAMETER_KEY_REQUEST_SYNC_FRAME, 0);
+                currentCodec.setParameters(params);
+            }
+        } catch (IllegalStateException e) {
+            Log.w(TAG, "Can't send keyframe request", e);
+        }
+    }
 
+    private static @NonNull Size chooseOptimalSize(@Nullable Size[] choices, int textureViewWidth, int textureViewHeight, int maxWidth, int maxHeight, Size target) {
+        if (choices == null)
+            return target;
         // Collect the supported resolutions that are at least as big as the preview Surface
         List<Size> bigEnough = new ArrayList<>();
         // Collect the supported resolutions that are smaller than the preview Surface
         List<Size> notBigEnough = new ArrayList<>();
-        int w = aspectRatio.getWidth();
-        int h = aspectRatio.getHeight();
+        int w = target.getWidth();
+        int h = target.getHeight();
         for (Size option : choices) {
+            Log.w(TAG, "supportedSize: " + option);
             if (option.getWidth() <= maxWidth && option.getHeight() <= maxHeight &&
                     option.getHeight() == option.getWidth() * h / w) {
                 if (option.getWidth() >= textureViewWidth &&
@@ -252,8 +308,28 @@ class CameraServiceCamera2 extends CameraService {
         }
     }
 
-    @Override
-    public void openCamera(Context context, VideoParams videoParams, Object surface, CameraListener listener) {
+    private static @NonNull Range<Integer> chooseOptimalFpsRange(Range<Integer>[] ranges) {
+        Range<Integer> range = null;
+        if (ranges != null && ranges.length > 0) {
+            for (Range<Integer> r : ranges) {
+                if (r.getUpper() > FPS_MAX)
+                    continue;
+                if (range != null) {
+                    int d = Math.abs(r.getUpper() - FPS_TARGET) - Math.abs(range.getUpper() - FPS_TARGET);
+                    if (d > 0)
+                        continue;
+                    if (d == 0 && r.getLower() > range.getLower())
+                        continue;
+                }
+                range = r;
+            }
+            if (range == null)
+                range = ranges[0];
+        }
+        return range == null ? new Range<>(FPS_TARGET, FPS_TARGET) : range;
+    }
+
+    public void openCamera(@NonNull Context context, VideoParams videoParams, TextureView surface, CameraListener listener, boolean hw_accel, boolean is_hd) {
         CameraDevice camera = previewCamera;
         if (camera != null) {
             camera.close();
@@ -265,44 +341,24 @@ class CameraServiceCamera2 extends CameraService {
             t.start();
         Handler handler = new Handler(t.getLooper());
         try {
-            CameraCharacteristics cc = manager.getCameraCharacteristics(videoParams.id);
-            StreamConfigurationMap streamConfigs = cc.get(CameraCharacteristics.SCALER_STREAM_CONFIGURATION_MAP);
-            cc.get(CameraCharacteristics.CONTROL_AE_AVAILABLE_TARGET_FPS_RANGES);
-            Size[] sizes = streamConfigs.getOutputSizes(SurfaceHolder.class);
-            for (Size s : sizes)
-                Log.w(TAG, "supportedSize: " + s);
             AutoFitTextureView view = (AutoFitTextureView) surface;
             boolean flip = videoParams.rotation % 180 != 0;
 
-            Size previewSize = chooseOptimalSize(sizes,
+            CameraCharacteristics cc = manager.getCameraCharacteristics(videoParams.id);
+            final Range<Integer> fpsRange = chooseOptimalFpsRange(cc.get(CameraCharacteristics.CONTROL_AE_AVAILABLE_TARGET_FPS_RANGES));
+
+            StreamConfigurationMap streamConfigs = cc.get(CameraCharacteristics.SCALER_STREAM_CONFIGURATION_MAP);
+            final Size previewSize = chooseOptimalSize(streamConfigs == null ? null : streamConfigs.getOutputSizes(SurfaceHolder.class),
                     flip ? view.getHeight() : view.getWidth(), flip ? view.getWidth() : view.getHeight(),
                     videoParams.width, videoParams.height,
                     new Size(videoParams.width, videoParams.height));
-
-            //videoParams.
-            Log.e(TAG, "openCamera " + videoParams.width + "x" + videoParams.height + " flip:" + flip);
-            Log.e(TAG, "openCamera " + videoParams.rotWidth + "x" + videoParams.rotHeight);
-            Log.e(TAG, "openCamera view " + view.getWidth() + "x" + view.getHeight());
-            Log.e(TAG, "openCamera previewSize " + previewSize.getWidth() + "x" + previewSize.getHeight());
-
-            //view.setAspectRatio(flip ? videoParams.height : videoParams.width, flip ? videoParams.width : videoParams.height);
-            //view.setAspectRatio(videoParams.height, videoParams.width);
-            int orientation = context.getResources().getConfiguration().orientation;
-            if (orientation == Configuration.ORIENTATION_LANDSCAPE) {
-                view.setAspectRatio(
-                        previewSize.getWidth(), previewSize.getHeight());
-            } else {
-                view.setAspectRatio(
-                        previewSize.getHeight(), previewSize.getWidth());
-            }
+            Log.d(TAG, "Selected preview size: " + previewSize + ", fps range: " + fpsRange + " rate: "+videoParams.rate);
+            view.setAspectRatio(previewSize.getHeight(), previewSize.getWidth());
 
             SurfaceTexture texture = view.getSurfaceTexture();
-            //texture.setDefaultBufferSize(flip ? previewSize.getHeight() : previewSize.getWidth(), flip ? previewSize.getWidth() : previewSize.getHeight());
-
             Surface s = new Surface(texture);
-            //s.setFixedSize(flip ? videoParams.height : videoParams.width, flip ? videoParams.width : videoParams.height);
 
-            Pair<MediaCodec, Surface> codec = USE_HARDWARE_ENCODER ? openCameraWithEncoder(videoParams, MediaFormat.MIMETYPE_VIDEO_VP8) : null;
+            final Pair<MediaCodec, Surface> codec = hw_accel ? openCameraWithEncoder(videoParams, videoParams.getCodec(), handler, is_hd) : null;
 
             final List<Surface> targets = new ArrayList<>(2);
             targets.add(s);
@@ -319,6 +375,7 @@ class CameraServiceCamera2 extends CameraService {
                 targets.add(tmpReader.getSurface());
             }
             final ImageReader reader = tmpReader;
+            final boolean[] codecStarted = {false};
 
             manager.openCamera(videoParams.id, new CameraDevice.StateCallback() {
                 @Override
@@ -331,12 +388,12 @@ class CameraServiceCamera2 extends CameraService {
                         builder.addTarget(s);
                         if (codec != null && codec.second != null) {
                             builder.addTarget(codec.second);
-                        } else {
+                        } else if (reader != null) {
                             builder.addTarget(reader.getSurface());
                         }
                         builder.set(CaptureRequest.CONTROL_AF_MODE, CaptureRequest.CONTROL_AF_MODE_CONTINUOUS_VIDEO);
                         builder.set(CaptureRequest.CONTROL_AE_MODE, CaptureRequest.CONTROL_AE_MODE_ON);
-                        builder.set(CaptureRequest.CONTROL_AE_TARGET_FPS_RANGE, new Range<>(0, 24));
+                        builder.set(CaptureRequest.CONTROL_AE_TARGET_FPS_RANGE, fpsRange);
                         builder.set(CaptureRequest.CONTROL_AWB_MODE, CaptureRequest.CONTROL_AWB_MODE_AUTO);
                         final CaptureRequest request = builder.build();
 
@@ -351,11 +408,16 @@ class CameraServiceCamera2 extends CameraService {
                                         public void onCaptureStarted(@NonNull CameraCaptureSession session, @NonNull CaptureRequest request, long timestamp, long frameNumber) {
                                             if (frameNumber == 1) {
                                                 codec.first.start();
+                                                codecStarted[0] = true;
                                             }
                                         }
                                     } : null, handler);
-                                } catch (CameraAccessException e) {
+                                    if (codec != null && codec.first != null) {
+                                        currentCodec = codec.first;
+                                    }
+                                } catch (Exception e) {
                                     Log.w(TAG, "onConfigured error:", e);
+                                    camera.close();
                                 }
                             }
 
@@ -364,6 +426,11 @@ class CameraServiceCamera2 extends CameraService {
                                 listener.onError();
                                 Log.w(TAG, "onConfigureFailed");
                             }
+
+                            @Override
+                            public void onClosed(@NonNull CameraCaptureSession session) {
+                                Log.w(TAG, "CameraCaptureSession onClosed");
+                            }
                         }, handler);
                     } catch (Exception e) {
                         Log.w(TAG, "onOpened error:", e);
@@ -373,33 +440,33 @@ class CameraServiceCamera2 extends CameraService {
                 @Override
                 public void onDisconnected(@NonNull CameraDevice camera) {
                     Log.w(TAG, "onDisconnected");
-                    if (previewCamera == camera) {
-                        previewCamera = null;
-                    }
                     camera.close();
-                    if (codec != null && codec.first != null) {
-                        codec.first.signalEndOfInputStream();
-                        codec.first.release();
-                    }
+                    listener.onError();
                 }
 
                 @Override
                 public void onError(@NonNull CameraDevice camera, int error) {
                     Log.w(TAG, "onError: " + error);
+                    camera.close();
+                    listener.onError();
+                }
+
+                @Override
+                public void onClosed(@NonNull CameraDevice camera) {
+                    Log.w(TAG, "onClosed");
                     if (previewCamera == camera)
                         previewCamera = null;
                     if (codec != null && codec.first != null) {
+                        if (codecStarted[0])
+                            codec.first.signalEndOfInputStream();
                         codec.first.release();
                     }
-                    listener.onError();
                 }
             }, handler);
-        } catch (CameraAccessException e) {
-            Log.e(TAG, "Error while settings preview parameters", e);
         } catch (SecurityException e) {
-            Log.e(TAG, "Error while settings preview parameters", e);
-        } catch (RuntimeException e) {
-            Log.e(TAG, "Error while settings preview parameters", e);
+            Log.e(TAG, "Security exception while settings preview parameters", e);
+        } catch (Exception e) {
+            Log.e(TAG, "Exception while settings preview parameters", e);
         }
     }
 
@@ -413,7 +480,7 @@ class CameraServiceCamera2 extends CameraService {
         if (manager == null)
             return;
         try {
-            CameraCharacteristics cc = manager.getCameraCharacteristics(camId);
+            final CameraCharacteristics cc = manager.getCameraCharacteristics(camId);
             StreamConfigurationMap streamConfigs = cc.get(CameraCharacteristics.SCALER_STREAM_CONFIGURATION_MAP);
             if (streamConfigs == null)
                 return;
@@ -450,18 +517,20 @@ class CameraServiceCamera2 extends CameraService {
     @Override
     public void closeCamera() {
         CameraDevice camera = previewCamera;
-        previewCamera = null;
-        camera.close();
+        if (camera != null) {
+            previewCamera = null;
+            camera.close();
+        }
     }
 
     @Override
     String[] getCameraIds() {
-            try {
-                return manager.getCameraIdList();
-            } catch (Exception e) {
-                Log.w(TAG, "getCameraIds: can't enumerate devices", e);
-                return new String[0];
-            }
+        try {
+            return manager.getCameraIdList();
+        } catch (Exception e) {
+            Log.w(TAG, "getCameraIds: can't enumerate devices", e);
+            return new String[0];
+        }
     }
 
     @Override
