@@ -19,18 +19,8 @@
 #import "RingWindowController.h"
 #import <QuartzCore/QuartzCore.h>
 #include <qrencode.h>
-#include <memory>
-
-//Qt
-#import <QItemSelectionModel>
-#import <QItemSelection>
 
 //LRC
-#import <callmodel.h>
-#import <account.h>
-#import <call.h>
-#import <recentmodel.h>
-#import <AvailableAccountModel.h>
 #import <api/lrc.h>
 #import <api/account.h>
 #import <api/newaccountmodel.h>
@@ -40,7 +30,7 @@
 #import <api/contactmodel.h>
 #import <api/contact.h>
 #import <api/datatransfermodel.h>
-#import <media/recordingmodel.h>
+#import <api/avmodel.h>
 
 // Ring
 #import "AppDelegate.h"
@@ -52,13 +42,23 @@
 #import "SmartViewVC.h"
 #import "views/IconButton.h"
 #import "views/NSColor+RingTheme.h"
-#import "views/BackgroundView.h"
-#import "ChooseAccountVC.h"
+#import "views/HoverButton.h"
 #import "utils.h"
+#import "RingWizardWC.h"
+#import "AccountSettingsVC.h"
 
-@interface RingWindowController () <MigrateRingAccountsDelegate, NSToolbarDelegate>
+typedef NS_ENUM(NSInteger, ViewState) {
+    SHOW_WELCOME_SCREEN = 0,
+    SHOW_CONVERSATION_SCREEN,
+    SHOW_CALL_SCREEN,
+    SHOW_SETTINGS_SCREEN,
+    LEAVE_MESSAGE,
+};
+
+@interface RingWindowController () <MigrateRingAccountsDelegate>
 
 @property (retain) MigrateRingAccountsWC* migrateWC;
+@property RingWizardWC* wizard;
 
 @end
 
@@ -70,82 +70,153 @@
     __unsafe_unretained IBOutlet NSView* welcomeContainer;
     __unsafe_unretained IBOutlet NSView* callView;
     __unsafe_unretained IBOutlet NSTextField* ringIDLabel;
+    __unsafe_unretained IBOutlet NSTextField* explanationLabel;
     __unsafe_unretained IBOutlet NSButton* shareButton;
     __unsafe_unretained IBOutlet NSImageView* qrcodeView;
 
-    std::unique_ptr<lrc::api::Lrc> lrc_;
-
     PreferencesWC* preferencesWC;
     IBOutlet SmartViewVC* smartViewVC;
 
     CurrentCallVC* currentCallVC;
     ConversationVC* conversationVC;
+    AccountSettingsVC* settingsVC;
 
-    // toolbar menu items
-    ChooseAccountVC* chooseAccountVC;
+    IBOutlet ChooseAccountVC* chooseAccountVC;
 }
 
-static NSString* const kPreferencesIdentifier        = @"PreferencesIdentifier";
-NSString* const kChangeAccountToolBarItemIdentifier  = @"ChangeAccountToolBarItemIdentifier";
+@synthesize dataTransferModel, accountModel, behaviorController, avModel;
+@synthesize wizard;
+
+-(id) initWithWindowNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil accountModel:( lrc::api::NewAccountModel*)accountModel dataTransferModel:( lrc::api::DataTransferModel*)dataTransferModel behaviourController:( lrc::api::BehaviorController*) behaviorController avModel: (lrc::api::AVModel*)avModel
+{
+    if (self =  [self initWithWindowNibName:nibNameOrNil])
+    {
+        self.accountModel = accountModel;
+        self.dataTransferModel = dataTransferModel;
+        self.behaviorController = behaviorController;
+        self.avModel = avModel;
+        self.avModel->useAVFrame(YES);
+    }
+    return self;
+}
+
+- (NSApplicationPresentationOptions)window:(NSWindow *)window willUseFullScreenPresentationOptions:(NSApplicationPresentationOptions)proposedOptions
+{
+    return (NSApplicationPresentationFullScreen |
+            NSApplicationPresentationAutoHideMenuBar |
+            NSApplicationPresentationAutoHideToolbar);
+}
+
+-(void)changeViewTo:(ViewState) state  {
+    switch (state) {
+        case SHOW_WELCOME_SCREEN:
+            [self accountSettingsShouldOpen: NO];
+            [conversationVC hideWithAnimation:false];
+            [currentCallVC hideWithAnimation:false];
+            [currentCallVC cleanUp];
+            [currentCallVC.view removeFromSuperview];
+            [welcomeContainer setHidden: NO];
+            [smartViewVC.view setHidden: NO];
+            [settingsVC hide];
+            break;
+        case SHOW_CONVERSATION_SCREEN:
+            [self accountSettingsShouldOpen: NO];
+            [conversationVC showWithAnimation:false];
+            [currentCallVC hideWithAnimation:false];
+            [currentCallVC cleanUp];
+            [currentCallVC.view removeFromSuperview];
+            [welcomeContainer setHidden: YES];
+            [smartViewVC.view setHidden: NO];
+            [settingsVC hide];
+            break;
+        case SHOW_CALL_SCREEN:
+            self.avModel->useAVFrame(YES);
+            [self accountSettingsShouldOpen: NO];
+            if (![currentCallVC.view superview]) {
+            [callView addSubview:[currentCallVC view] positioned:NSWindowAbove relativeTo:nil];
+            [currentCallVC initFrame];
+            [currentCallVC showWithAnimation:false];
+            [conversationVC hideWithAnimation:false];
+            [welcomeContainer setHidden: YES];
+            [smartViewVC.view setHidden: NO];
+            [settingsVC hide];
+            }
+            [currentCallVC showWithAnimation:false];
+            break;
+        case SHOW_SETTINGS_SCREEN:
+            @try {
+                [self accountSettingsShouldOpen: YES];
+            }
+            @catch (NSException *ex) {
+                return;
+            }
+            [welcomeContainer setHidden: YES];
+            [currentCallVC hideWithAnimation:false];
+            [currentCallVC.view removeFromSuperview];
+            [conversationVC hideWithAnimation:false];
+            [smartViewVC.view setHidden: YES];
+            [settingsVC show];
+            break;
+        case LEAVE_MESSAGE:
+            [conversationVC showWithAnimation: false];
+            [currentCallVC hideWithAnimation: false];
+            [conversationVC presentLeaveMessageView];
+        default:
+            break;
+    }
+}
 
 - (void)windowDidLoad {
     [super windowDidLoad];
     [self.window setMovableByWindowBackground:YES];
 
-    [self.window setBackgroundColor:[NSColor colorWithRed:242.0/255 green:242.0/255 blue:242.0/255 alpha:1.0]];
     self.window.titleVisibility = NSWindowTitleHidden;
 
-    lrc_ = std::make_unique<lrc::api::Lrc>();
-
     currentCallVC = [[CurrentCallVC alloc] initWithNibName:@"CurrentCall" bundle:nil];
-    conversationVC = [[ConversationVC alloc] initWithNibName:@"Conversation" bundle:nil delegate:self];
-    // toolbar items
-    chooseAccountVC = [[ChooseAccountVC alloc] initWithNibName:@"ChooseAccount" bundle:nil model:&(lrc_->getAccountModel()) delegate:self];
+    currentCallVC.delegate = self;
+    conversationVC = [[ConversationVC alloc] initWithNibName:@"Conversation" bundle:nil delegate:self aVModel:self.avModel];
+    [chooseAccountVC updateWithDelegate: self andModel:self.accountModel];
+    settingsVC = [[AccountSettingsVC alloc] initWithNibName:@"AccountSettings" bundle:nil accountmodel:self.accountModel];
     [callView setAutoresizingMask:NSViewWidthSizable | NSViewHeightSizable];
     [[currentCallVC view] setAutoresizingMask:NSViewWidthSizable | NSViewHeightSizable];
     [[conversationVC view] setAutoresizingMask:NSViewWidthSizable | NSViewHeightSizable];
+    [[settingsVC view] setAutoresizingMask:NSViewWidthSizable | NSViewHeightSizable];
 
-    [callView addSubview:[currentCallVC view] positioned:NSWindowAbove relativeTo:nil];
     [callView addSubview:[conversationVC view] positioned:NSWindowAbove relativeTo:nil];
+    [self.window.contentView addSubview:[settingsVC view] positioned:NSWindowAbove relativeTo:nil];
 
-    [currentCallVC initFrame];
     [conversationVC initFrame];
-    @try {
-        [smartViewVC setConversationModel: [chooseAccountVC selectedAccount].conversationModel.get()];
-    }
-    @catch (NSException *ex) {
-        NSLog(@"Caught exception %@: %@", [ex name], [ex reason]);
-    }
+    [settingsVC initFrame];
 
-    // Fresh run, we need to make sure RingID appears
-    [shareButton sendActionOn:NSLeftMouseDownMask];
+    [self checkAccountsToMigrate];
 
-    [self connect];
-    [self updateRingID];
-    // display accounts to select
-    NSToolbar *toolbar = self.window.toolbar;
-    toolbar.delegate = self;
-    [toolbar insertItemWithItemIdentifier:kChangeAccountToolBarItemIdentifier atIndex:1];
     // set download folder (default - 'Documents')
     NSString* path = [[NSUserDefaults standardUserDefaults] stringForKey:Preferences::DownloadFolder];
     if (!path || path.length == 0) {
         NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
         path = [[paths objectAtIndex:0] stringByAppendingString:@"/"];
     }
-    lrc_->getDataTransferModel().downloadDirectory = std::string([path UTF8String]);
+    self.dataTransferModel->downloadDirectory = std::string([path UTF8String]);
     if(appSandboxed()) {
         NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
-        Media::RecordingModel::instance().setRecordPath(QString::fromNSString([paths objectAtIndex:0]));
+        avModel->setRecordPath([[paths objectAtIndex:0] UTF8String]);
     }
+    NSToolbar *tb = [[self window] toolbar];
+    [tb setAllowsUserCustomization:NO];
+
+    //add messages view controller to responders chain
+    NSResponder * viewNextResponder = [self nextResponder];
+    [self setNextResponder: [conversationVC getMessagesView]];
+    [[conversationVC getMessagesView] setNextResponder: viewNextResponder];
 }
 
 - (void) connect
 {
-    QObject::connect(&lrc_->getBehaviorController(),
+    QObject::connect(self.behaviorController,
                      &lrc::api::BehaviorController::showCallView,
                      [self](const std::string accountId,
                             const lrc::api::conversation::Info convInfo){
-                         auto* accInfo = &lrc_->getAccountModel().getAccountInfo(accountId);
+                         auto* accInfo = &self.accountModel->getAccountInfo(accountId);
                          if (accInfo->contactModel->getContact(convInfo.participants[0]).profileInfo.type == lrc::api::profile::Type::PENDING)
                              [smartViewVC selectPendingList];
                          else
@@ -153,17 +224,17 @@ NSString* const kChangeAccountToolBarItemIdentifier  = @"ChangeAccountToolBarIte
 
                          [currentCallVC setCurrentCall:convInfo.callId
                                           conversation:convInfo.uid
-                                               account:accInfo];
-                         [smartViewVC selectConversation: convInfo model:accInfo->conversationModel.get()];
-                         [currentCallVC showWithAnimation:false];
-                         [conversationVC hideWithAnimation:false];
+                                               account:accInfo
+                                               avModel: avModel];
+                         [self changeViewTo:SHOW_CALL_SCREEN];
+
                      });
 
-    QObject::connect(&lrc_->getBehaviorController(),
+    QObject::connect(self.behaviorController,
                      &lrc::api::BehaviorController::showIncomingCallView,
                      [self](const std::string accountId,
                             const lrc::api::conversation::Info convInfo){
-                         auto* accInfo = &lrc_->getAccountModel().getAccountInfo(accountId);
+                         auto* accInfo = &self.accountModel->getAccountInfo(accountId);
                          if (accInfo->contactModel->getContact(convInfo.participants[0]).profileInfo.type == lrc::api::profile::Type::PENDING)
                              [smartViewVC selectPendingList];
                          else
@@ -171,22 +242,71 @@ NSString* const kChangeAccountToolBarItemIdentifier  = @"ChangeAccountToolBarIte
 
                          [currentCallVC setCurrentCall:convInfo.callId
                                           conversation:convInfo.uid
-                                               account:accInfo];
+                                               account:accInfo
+                                               avModel: avModel];
                          [smartViewVC selectConversation: convInfo model:accInfo->conversationModel.get()];
-                         [currentCallVC showWithAnimation:false];
-                         [conversationVC hideWithAnimation:false];
+                         [self changeViewTo:SHOW_CALL_SCREEN];
                      });
 
-    QObject::connect(&lrc_->getBehaviorController(),
+    QObject::connect(self.behaviorController,
                      &lrc::api::BehaviorController::showChatView,
                      [self](const std::string& accountId,
                             const lrc::api::conversation::Info& convInfo){
-                         auto& accInfo = lrc_->getAccountModel().getAccountInfo(accountId);
+                         auto& accInfo = self.accountModel->getAccountInfo(accountId);
                          [conversationVC setConversationUid:convInfo.uid model:accInfo.conversationModel.get()];
                          [smartViewVC selectConversation: convInfo model:accInfo.conversationModel.get()];
-                         [conversationVC showWithAnimation:false];
-                         [currentCallVC hideWithAnimation:false];
+                         [self changeViewTo:SHOW_CONVERSATION_SCREEN];
                      });
+    QObject::connect(self.behaviorController,
+                     &lrc::api::BehaviorController::showLeaveMessageView,
+                     [self](const std::string& accountId,
+                            const lrc::api::conversation::Info& convInfo){
+                         auto& accInfo = self.accountModel->getAccountInfo(accountId);
+                         [conversationVC setConversationUid:convInfo.uid model:accInfo.conversationModel.get()];
+                         [smartViewVC selectConversation: convInfo model:accInfo.conversationModel.get()];
+                         [self changeViewTo:LEAVE_MESSAGE];
+                     });
+}
+
+#pragma mark - Ring account migration
+
+- (void) migrateRingAccount:(std::string) acc
+{
+    self.migrateWC = [[MigrateRingAccountsWC alloc] initWithDelegate:self actionCode:1];
+    self.migrateWC.accountModel = self.accountModel;
+    self.migrateWC.accountToMigrate = acc;
+    [self.window beginSheet:self.migrateWC.window completionHandler:nil];
+}
+
+- (void)checkAccountsToMigrate
+{
+    auto accounts = self.accountModel->getAccountList();
+    for (auto accountId: accounts) {
+        const lrc::api::account::Info& accountInfo = self.accountModel->getAccountInfo(accountId);
+        if (accountInfo.status == lrc::api::account::Status::ERROR_NEED_MIGRATION) {
+            [self migrateRingAccount:accountInfo.id];
+            return;
+        }
+    }
+    @try {
+        [smartViewVC setConversationModel: [chooseAccountVC selectedAccount].conversationModel.get()];
+    }
+    @catch (NSException *ex) {
+        NSLog(@"Caught exception %@: %@", [ex name], [ex reason]);
+    }
+    [shareButton sendActionOn:NSLeftMouseDownMask];
+    [self connect];
+    [self updateRingID];
+}
+
+- (void)migrationDidComplete
+{
+    [self checkAccountsToMigrate];
+}
+
+- (void)migrationDidCompleteWithError
+{
+    [self checkAccountsToMigrate];
 }
 
 /**
@@ -213,8 +333,10 @@ NSString* const kChangeAccountToolBarItemIdentifier  = @"ChangeAccountToolBarIte
         NSString* uriToDisplay = nullptr;
         if (!registeredName.empty()) {
             uriToDisplay = @(registeredName.c_str());
+            [explanationLabel setStringValue: NSLocalizedString(@"This is your Jami username. \nCopy and share it with your friends!", @"Explanation label when user have Jami username")];
         } else {
             uriToDisplay = @(ringID.c_str());
+            [explanationLabel setStringValue: NSLocalizedString(@"This is your ID. \nCopy and share it with your friends!", @"Explanation label when user have just ID")];
         }
         [ringIDLabel setStringValue:uriToDisplay];
         [self drawQRCode:@(ringID.c_str())];
@@ -238,8 +360,9 @@ NSString* const kChangeAccountToolBarItemIdentifier  = @"ChangeAccountToolBarIte
 
 - (IBAction)toggleQRCode:(id)sender {
     // Toggle pressed state of QRCode button
-    [sender setPressed:![sender isPressed]];
-    [self showQRCode:[sender isPressed]];
+  //  [sender setPressed:![sender isPressed]];
+    bool show = qrcodeView.animator.alphaValue == 0.0f ? YES: NO;
+    [self showQRCode: show];
 }
 
 /**
@@ -279,7 +402,7 @@ NSString* const kChangeAccountToolBarItemIdentifier  = @"ChangeAccountToolBarIte
     for(int i = 0; i < width; ++i) {
         for(int j = 0; j < width; ++j) {
             if(*data & 1) {
-                CGContextSetFillColorWithColor(ctx, [NSColor ringDarkGrey].CGColor);
+                CGContextSetFillColorWithColor(ctx, [NSColor labelColor].CGColor);
                 rectDraw.origin = CGPointMake((j + qr_margin) * zoom,(i + qr_margin) * zoom);
                 CGContextAddRect(ctx, rectDraw);
                 CGContextFillPath(ctx);
@@ -328,7 +451,8 @@ NSString* const kChangeAccountToolBarItemIdentifier  = @"ChangeAccountToolBarIte
         [preferencesWC.window orderFront:preferencesWC.window];
         return;
     }
-    preferencesWC = [[PreferencesWC alloc] initWithNibName:@"PreferencesWindow" bundle: nil model:&(lrc_->getDataTransferModel())];
+
+    preferencesWC = [[PreferencesWC alloc] initWithWindowNibName: @"PreferencesWindow" bundle: nil accountModel:self.accountModel dataTransferModel:self.dataTransferModel behaviourController:self.behaviorController avModel: self.avModel];
     [preferencesWC.window makeKeyAndOrderFront:preferencesWC.window];
 }
 
@@ -345,76 +469,33 @@ NSString* const kChangeAccountToolBarItemIdentifier  = @"ChangeAccountToolBarIte
     }
 }
 
-#pragma mark - Ring account migration
-
-- (void) migrateRingAccount:(Account*) acc
-{
-    self.migrateWC = [[MigrateRingAccountsWC alloc] initWithDelegate:self actionCode:1];
-    self.migrateWC.account = acc;
-#if MAC_OS_X_VERSION_MIN_REQUIRED > MAC_OS_X_VERSION_10_9
-    [self.window beginSheet:self.migrateWC.window completionHandler:nil];
-#else
-    [NSApp beginSheet: self.migrateWC.window
-       modalForWindow: self.window
-        modalDelegate: self
-       didEndSelector: nil
-          contextInfo: nil];
-#endif
-}
-
-// TODO: Reimplement as a blocking loop when new LRC models handle migration
-- (void)checkAccountsToMigrate
-{
-    auto ringList = AccountModel::instance().accountsToMigrate();
-    if (ringList.length() > 0){
-        Account* acc = ringList.value(0);
-        [self migrateRingAccount:acc];
-    } else {
-        // Fresh run, we need to make sure RingID appears
-        [shareButton sendActionOn:NSLeftMouseDownMask];
-
-        [self connect];
-        [self updateRingID];
-        // display accounts to select
-        NSToolbar *toolbar = self.window.toolbar;
-        toolbar.delegate = self;
-        [toolbar insertItemWithItemIdentifier:kChangeAccountToolBarItemIdentifier atIndex:1];
-    }
-}
-
-- (void)migrationDidComplete
-{
-    [self checkAccountsToMigrate];
-}
-
-- (void)migrationDidCompleteWithError
-{
-    [self checkAccountsToMigrate];
-}
-
--(void)selectAccount:(const lrc::api::account::Info&)accInfo
+- (void) selectAccount:(const lrc::api::account::Info&)accInfo currentRemoved:(BOOL) removed
 {
     // If the selected account has been changed, we close any open panel
-    if ([smartViewVC setConversationModel:accInfo.conversationModel.get()]) {
-        [currentCallVC hideWithAnimation:false];
-        [conversationVC hideWithAnimation:false];
-    }
-
+    [smartViewVC setConversationModel:accInfo.conversationModel.get()];
     // Welcome view informations are also updated
     [self updateRingID];
+    [settingsVC setSelectedAccount:accInfo.id];
+    [self changeViewTo: ([settingsVC.view isHidden] || removed)  ?  SHOW_WELCOME_SCREEN : SHOW_SETTINGS_SCREEN];
 }
 
 -(void)allAccountsDeleted
 {
     [smartViewVC clearConversationModel];
-    [currentCallVC hideWithAnimation:false];
-    [conversationVC hideWithAnimation:false];
+    [self changeViewTo:SHOW_WELCOME_SCREEN];
     [self updateRingID];
+    qrcodeView.animator.alphaValue = 0.0;
+    [centerYQRCodeConstraint.animator setConstant: 0];
+    [centerYWelcomeContainerConstraint.animator setConstant: 0];
+    [self close];
+    AppDelegate* delegate = (AppDelegate*)[[NSApplication sharedApplication] delegate];
+    [delegate showWizard];
 }
 
 -(void)rightPanelClosed
 {
     [smartViewVC deselect];
+    [welcomeContainer setHidden:NO];
 }
 
 -(void)currentConversationTrusted
@@ -423,19 +504,98 @@ NSString* const kChangeAccountToolBarItemIdentifier  = @"ChangeAccountToolBarIte
 }
 
 -(void) listTypeChanged {
-    [conversationVC hideWithAnimation:false];
-    [currentCallVC hideWithAnimation:false];
+    [self changeViewTo:SHOW_WELCOME_SCREEN];
 }
 
-#pragma mark - NSToolbarDelegate
-- (nullable NSToolbarItem *)toolbar:(NSToolbar *)toolbar itemForItemIdentifier:(NSString *)itemIdentifier willBeInsertedIntoToolbar:(BOOL)flag
+- (IBAction)openAccountSettings:(NSButton *)sender
 {
-    if(itemIdentifier == kChangeAccountToolBarItemIdentifier) {
-        NSToolbarItem *toolbarItem = [[NSToolbarItem alloc] initWithItemIdentifier:kChangeAccountToolBarItemIdentifier];
-        toolbarItem.view = chooseAccountVC.view;
-        return toolbarItem;
-    }
-    return nil;
+    [self changeViewTo: [settingsVC.view isHidden] ?  SHOW_SETTINGS_SCREEN : SHOW_WELCOME_SCREEN];
 }
 
+- (void) createNewAccount {
+    [self changeViewTo:SHOW_WELCOME_SCREEN];
+    wizard = [[RingWizardWC alloc] initWithNibName:@"RingWizard" bundle: nil accountmodel: self.accountModel];
+    [wizard showChooseWithCancelButton: YES andAdvanced: YES];
+    [self.window beginSheet:wizard.window completionHandler:nil];
+}
+
+- (NSRect)window:(NSWindow *)window willPositionSheet:(NSWindow *)sheet
+       usingRect:(NSRect)rect
+{
+    float titleBarHeight = self.window.frame.size.height - [NSWindow contentRectForFrameRect:self.window.frame styleMask:self.window.styleMask].size.height;
+    rect.origin.y = self.window.frame.size.height;
+    return rect;
+}
+
+-(void) accountSettingsShouldOpen: (BOOL) open {
+    if (open) {
+        [settingsVC setSelectedAccount: [chooseAccountVC selectedAccount].id];
+    }
+}
+
+#pragma mark - CallViewControllerDelegate
+
+-(void) conversationInfoUpdatedFor:(const std::string&) conversationID {
+    [smartViewVC reloadConversationWithUid:@(conversationID.c_str())];
+}
+
+-(void) showConversation:(NSString* )conversationId forAccount:(NSString*)accountId {
+    auto& accInfo = self.accountModel->getAccountInfo([accountId UTF8String]);
+    [chooseAccountVC selectAccount: accountId];
+    [settingsVC setSelectedAccount: [accountId UTF8String]];
+    [smartViewVC setConversationModel:accInfo.conversationModel.get()];
+    [smartViewVC selectConversationList];
+    [self updateRingID];
+    auto convInfo = getConversationFromUid([conversationId UTF8String], *accInfo.conversationModel.get());
+    auto convQueue = accInfo.conversationModel.get()->allFilteredConversations();
+    if (convInfo != convQueue.end()) {
+        [conversationVC setConversationUid:convInfo->uid model:accInfo.conversationModel.get()];
+        [smartViewVC selectConversation: *convInfo model:accInfo.conversationModel.get()];
+        accInfo.conversationModel.get()->clearUnreadInteractions([conversationId UTF8String]);
+    }
+    [self changeViewTo:SHOW_CONVERSATION_SCREEN];
+}
+
+-(void) showCall:(NSString* )callId forAccount:(NSString*)accountId forConversation:(NSString*)conversationId {
+    auto& accInfo = self.accountModel->getAccountInfo([accountId UTF8String]);
+    [chooseAccountVC selectAccount: accountId];
+    [settingsVC setSelectedAccount:accInfo.id];
+    [smartViewVC setConversationModel:accInfo.conversationModel.get()];
+    [self updateRingID];
+    auto convInfo = getConversationFromUid([conversationId UTF8String], *accInfo.conversationModel.get());
+    auto convQueue = accInfo.conversationModel.get()->allFilteredConversations();
+    if (convInfo != convQueue.end()) {
+        if (accInfo.contactModel->getContact(convInfo->participants[0]).profileInfo.type == lrc::api::profile::Type::PENDING)
+            [smartViewVC selectPendingList];
+        else
+            [smartViewVC selectConversationList];
+        [smartViewVC selectConversation: *convInfo model:accInfo.conversationModel.get()];
+    }
+    [currentCallVC setCurrentCall:[callId UTF8String]
+                     conversation:[conversationId UTF8String]
+                          account:&accInfo
+                          avModel:avModel];
+    [self changeViewTo:SHOW_CALL_SCREEN];
+}
+
+-(void) showContactRequestFor:(NSString* )accountId contactUri:(NSString*)uri {
+    auto& accInfo = self.accountModel->getAccountInfo([accountId UTF8String]);
+    [chooseAccountVC selectAccount: accountId];
+    [settingsVC setSelectedAccount:accInfo.id];
+    [smartViewVC setConversationModel:accInfo.conversationModel.get()];
+    [self updateRingID];
+    [smartViewVC selectPendingList];
+    auto convInfo = getConversationFromURI([uri UTF8String], *accInfo.conversationModel.get());
+    auto convQueue = accInfo.conversationModel.get()->allFilteredConversations();
+    if (convInfo != convQueue.end()) {
+        [conversationVC setConversationUid:convInfo->uid model:accInfo.conversationModel.get()];
+        [smartViewVC selectConversation: *convInfo model:accInfo.conversationModel.get()];
+    }
+    [self changeViewTo:SHOW_CONVERSATION_SCREEN];
+}
+
+- (BOOL)windowShouldClose:(id)sender {
+    [NSApp hide:nil];
+    return NO;
+}
 @end
