@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2015-2017 Savoir-faire Linux Inc.
+ *  Copyright (C) 2015-2019 Savoir-faire Linux Inc.
  *
  *  Author: Eloi BAIL <eloi.bail@savoirfairelinux.com>
  *  Author: Adrien Béraud <adrien.beraud@savoirfairelinux.com>
@@ -19,24 +19,23 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA.
  */
 
-#ifndef __MEDIA_CODEC_H__
-#define __MEDIA_CODEC_H__
+#pragma once
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
 #include "audio/audiobuffer.h" // for AudioFormat
-#include <cctype>
 #include "ip_utils.h"
 
+#include <cctype>
 #include <string>
 #include <vector>
 #include <map>
 #include <iostream>
 #include <unistd.h>
 
-namespace ring {
+namespace jami {
 
 enum CodecType : unsigned {
     CODEC_NONE = 0, // indicates that no codec is used or defined
@@ -60,22 +59,22 @@ enum MediaType : unsigned {
 struct SystemCodecInfo
 {
     static constexpr unsigned DEFAULT_CODEC_QUALITY {30};
-#ifdef RING_VIDEO
-    static constexpr unsigned DEFAULT_H264_MIN_QUALITY {35};
-    static constexpr unsigned DEFAULT_H264_MAX_QUALITY {25};
+#ifdef ENABLE_VIDEO
+    static constexpr unsigned DEFAULT_H264_MIN_QUALITY {40};
+    static constexpr unsigned DEFAULT_H264_MAX_QUALITY {20};
     static constexpr unsigned DEFAULT_VP8_MIN_QUALITY {50};
     static constexpr unsigned DEFAULT_VP8_MAX_QUALITY {20};
-    static constexpr unsigned DEFAULT_VIDEO_BITRATE {1000}; // in Kbits/second
+    static constexpr unsigned DEFAULT_VIDEO_BITRATE {1200}; // in Kbits/second
 #endif
 
     // indicates that the codec does not use quality factor
     static constexpr unsigned DEFAULT_NO_QUALITY {0};
 
-    static constexpr unsigned DEFAULT_MIN_BITRATE {250};
-    static constexpr unsigned DEFAULT_MAX_BITRATE {3000};
+    static constexpr unsigned DEFAULT_MIN_BITRATE {220};
+    static constexpr unsigned DEFAULT_MAX_BITRATE {6000};
 
-    SystemCodecInfo(unsigned avcodecId, const std::string name,
-                    std::string libName, MediaType mediaType,
+    SystemCodecInfo(unsigned avcodecId, const std::string& name,
+                    const std::string& libName, MediaType mediaType,
                     CodecType codecType = CODEC_NONE, unsigned bitrate = 0,
                     unsigned payloadType = 0,
                     unsigned m_minQuality = DEFAULT_NO_QUALITY,
@@ -107,8 +106,8 @@ struct SystemCodecInfo
  */
 struct SystemAudioCodecInfo : SystemCodecInfo
 {
-    SystemAudioCodecInfo(unsigned avcodecId, const std::string name,
-                         std::string libName, CodecType type,
+    SystemAudioCodecInfo(unsigned avcodecId, const std::string& name,
+                         const std::string& libName, CodecType type,
                          unsigned bitrate = 0,
                          unsigned sampleRate = 0, unsigned nbChannels = 0,
                          unsigned payloadType = 0);
@@ -127,8 +126,8 @@ struct SystemAudioCodecInfo : SystemCodecInfo
  */
 struct SystemVideoCodecInfo : SystemCodecInfo
 {
-    SystemVideoCodecInfo(unsigned avcodecId, const std::string name,
-                         std::string libName, CodecType type = CODEC_NONE,
+    SystemVideoCodecInfo(unsigned avcodecId, const std::string& name,
+                         const std::string& libName, CodecType type = CODEC_NONE,
                          unsigned bitrate = 0,
                          unsigned m_minQuality = 0,
                          unsigned m_maxQuality = 0,
@@ -154,19 +153,18 @@ struct AccountCodecInfo
 {
     AccountCodecInfo(const SystemCodecInfo& sysCodecInfo) noexcept;
     AccountCodecInfo(const AccountCodecInfo&) noexcept = default;
-    AccountCodecInfo(AccountCodecInfo&&) noexcept = default;
+    AccountCodecInfo(AccountCodecInfo&&) noexcept = delete;
     AccountCodecInfo& operator=(const AccountCodecInfo&);
-    AccountCodecInfo& operator=(AccountCodecInfo&&);
+    AccountCodecInfo& operator=(AccountCodecInfo&&) noexcept = delete;
 
     const SystemCodecInfo& systemCodecInfo;
-    unsigned order {0}; /*used to define prefered codec list order in UI*/
+    unsigned order {0}; /*used to define preferred codec list order in UI*/
     bool isActive {true};
     /* account custom values */
     unsigned payloadType;
     unsigned bitrate;
     unsigned quality;
     std::map<std::string, std::string>  getCodecSpecifications();
-
 };
 
 struct AccountAudioCodecInfo : AccountCodecInfo
@@ -192,7 +190,7 @@ struct AccountVideoCodecInfo : AccountCodecInfo
     unsigned frameRate;
     unsigned profileId;
     std::string parameters;
-    bool isAutoQualityEnabled{false};
+    bool isAutoQualityEnabled{true};
 };
 bool operator== (SystemCodecInfo codec1, SystemCodecInfo codec2);
 
@@ -285,5 +283,4 @@ struct MediaDescription {
     CryptoAttribute crypto {};
 };
 
-}//namespace ring
-#endif // __MEDIA_CODEC_H__
+}//namespace jami
